@@ -1,5 +1,5 @@
 /* rndegd.c  -	interface to the EGD
- *	Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -19,6 +19,9 @@
  */
 
 #include <config.h>
+
+#ifdef USE_RNDEGD
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -31,54 +34,18 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include "types.h"
-#include "g10lib.h"
-#ifndef IS_MODULE
+#include "util.h"
 #include "ttyio.h"
-#endif
-#include "dynload.h"
+#include "algorithms.h"
 #include "cipher.h"
+#include "i18n.h"
+
 
 #ifndef offsetof
 #define offsetof(type, member) ((size_t) &((type *)0)->member)
 #endif
 
-
-/* FIXME: this is duplicated code from util/fileutil
- * I don't think that this code should go into libgcrypt anyway.
- */
-char *
-my_make_filename( const char *first_part, ... )
-{
-    va_list arg_ptr ;
-    size_t n;
-    const char *s;
-    char *name, *home, *p;
-
-    va_start( arg_ptr, first_part ) ;
-    n = strlen(first_part)+1;
-    while( (s=va_arg(arg_ptr, const char *)) )
-	n += strlen(s) + 1;
-    va_end(arg_ptr);
-
-    home = NULL;
-    if( *first_part == '~' && first_part[1] == '/'
-			   && (home = getenv("HOME")) && *home )
-	n += strlen(home);
-
-    name = m_alloc(n);
-    p = home ? stpcpy(stpcpy(name,home), first_part+1)
-	     : stpcpy(name, first_part);
-    va_start( arg_ptr, first_part ) ;
-    while( (s=va_arg(arg_ptr, const char *)) )
-	p = stpcpy(stpcpy(p,"/"), s);
-    va_end(arg_ptr);
-
-    return name;
-}
-
-
-
-
+static int egd_socket = -1;
 
 static int
 do_write( int fd, void *buf, size_t nbytes )
@@ -102,19 +69,81 @@ do_write( int fd, void *buf, size_t nbytes )
 static int
 do_read( int fd, void *buf, size_t nbytes )
 {
-    int n, nread = 0;
-
-    do {
-	do {
-	    n = read(fd, (char*)buf + nread, nbytes );
-	} while( n == -1 && errno == EINTR );
-	if( n == -1 )
-	    return -1;
-	nread += n;
-    } while( nread < nbytes );
-    return nbytes;
+  int n, nread = 0;
+  
+  while (nbytes)
+    {
+      do {
+        n = read(fd, (char*)buf + nread, nbytes );
+      } while( n == -1 && errno == EINTR );
+      if( n == -1 )
+        return nread? nread:-1;
+      else if( n == 0 ) {
+        /* EGD probably died. */
+        errno = ECONNRESET;
+        return -1;
+      }
+      nread += n;
+      nbytes -= n;
+    } 
+  return nread;
 }
 
+/* Connect to the EGD and return the file descriptor.  Return -1 on
+   error.  With NOFAIL set to true, silently fail and return the
+   error, otherwise print an error message and die. */
+int
+rndegd_connect_socket (int nofail)
+{
+  int fd;
+  const char *bname = NULL;
+  char *name;
+  struct sockaddr_un addr;
+  int addr_len;
+
+  if (egd_socket != -1)
+    {
+      close (egd_socket);
+      egd_socket = -1;
+    }
+
+#ifdef EGD_SOCKET_NAME
+  bname = EGD_SOCKET_NAME;
+#endif
+  if ( !bname || !*bname )
+    bname = "=entropy";
+  
+  if ( *bname == '=' && bname[1] )
+    name = make_filename( g10_opt_homedir, bname+1 , NULL );
+  else
+    name = make_filename( bname , NULL );
+  
+  if ( strlen(name)+1 >= sizeof addr.sun_path ) 
+    g10_log_fatal ("EGD socketname is too long\n");
+  
+  memset( &addr, 0, sizeof addr );
+  addr.sun_family = AF_UNIX;
+  strcpy( addr.sun_path, name );	  
+  addr_len = (offsetof( struct sockaddr_un, sun_path )
+              + strlen( addr.sun_path ));
+  
+  fd = socket(AF_UNIX, SOCK_STREAM, 0);
+  if (fd == -1 && !nofail)
+    g10_log_fatal("can't create unix domain socket: %s\n",
+                  strerror(errno) );
+  else if (connect (fd, (struct sockaddr*)&addr, addr_len) == -1)
+    {
+      if (!nofail)
+        g10_log_fatal("can't connect to `%s': %s\n",
+                      name, strerror(errno) );
+      close (fd);
+      fd = -1;
+    }
+  m_free(name);
+  if (fd != -1)
+    egd_socket = fd;
+  return fd;
+}
 
 
 /****************
@@ -125,11 +154,11 @@ do_read( int fd, void *buf, size_t nbytes )
  * Using a level of 0 should never block and better add nothing
  * to the pool.  So this is just a dummy for EGD.
  */
-static int
-gather_random( void (*add)(const void*, size_t, int), int requester,
+int
+rndegd_gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level )
 {
-    static int fd = -1;
+    int fd = egd_socket;
     int n;
     byte buffer[256+2];
     int nbytes;
@@ -141,33 +170,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	return 0;
 
   restart:
-    if( do_restart ) {
-	if( fd != -1 ) {
-	    close( fd );
-	    fd = -1;
-	}
-    }
-    if( fd == -1 ) {
-	#warning Fixme: make the filename configurable
-	char *name = my_make_filename( "~/.gnupg-test", "entropy", NULL );
-	struct sockaddr_un addr;
-	int addr_len;
+    if (fd == -1 || do_restart)
+      fd = rndegd_connect_socket (0);
 
-	memset( &addr, 0, sizeof addr );
-	addr.sun_family = AF_UNIX;
-	strcpy( addr.sun_path, name );	  /* fixme: check that it is long enough */
-	addr_len = offsetof( struct sockaddr_un, sun_path )
-		   + strlen( addr.sun_path );
-
-	fd = socket(AF_UNIX, SOCK_STREAM, 0);
-	if( fd == -1 )
-	    g10_log_fatal("can't create unix domain socket: %s\n",
-							    strerror(errno) );
-	if( connect( fd, (struct sockaddr*)&addr, addr_len) == -1 )
-	    g10_log_fatal("can't connect to `%s': %s\n",
-						    name, strerror(errno) );
-	g10_free(name);
-    }
     do_restart = 0;
 
     nbytes = length < 255? length : 255;
@@ -195,11 +200,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
     }
 
     if( length ) {
-      #ifdef IS_MODULE
-	fprintf( stderr,
-      #else
 	tty_printf(
-      #endif
 	 _("Please wait, entropy is being gathered. Do some work if it would\n"
 	   "keep you from getting bored, because it will improve the quality\n"
 	   "of the entropy.\n") );
@@ -220,56 +221,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	(*add)( buffer, n, requester );
 	length -= n;
     }
-    memset(buffer, 0, sizeof(buffer) );
+    wipememory(buffer, sizeof(buffer) );
 
     return 0; /* success */
 }
 
-
-
-#ifndef IS_MODULE
-static
-#endif
-const char * const gnupgext_version = "RNDEGD ($Revision: 1.12 $)";
-
-static struct {
-    int class;
-    int version;
-    void *func;
-} func_table[] = {
-    { 40, 1, gather_random },
-};
-
-
-#ifndef IS_MODULE
-static
-#endif
-void *
-gnupgext_enum_func( int what, int *sequence, int *class, int *vers )
-{
-    void *ret;
-    int i = *sequence;
-
-    do {
-	if ( i >= DIM(func_table) || i < 0 ) {
-	    return NULL;
-	}
-	*class = func_table[i].class;
-	*vers  = func_table[i].version;
-	ret = func_table[i].func;
-	i++;
-    } while ( what && what != *class );
-
-    *sequence = i;
-    return ret;
-}
-
-#ifndef IS_MODULE
-void
-rndegd_constructor(void)
-{
-    register_internal_cipher_extension( gnupgext_version,
-					gnupgext_enum_func );
-}
-#endif
-
+#endif /*USE_RNDEGD*/
