@@ -137,7 +137,12 @@ do_add_key_flags (PKT_signature *sig, unsigned int use)
 
     buf[0] = 0;
     if (use & PUBKEY_USAGE_SIG)
-        buf[0] |= 0x01 | 0x02;
+      {
+	if(sig->sig_class==0x18)
+	  buf[0] |= 0x02; /* Don't set the certify flag for subkeys */
+	else
+	  buf[0] |= 0x01 | 0x02;
+      }
     if (use & PUBKEY_USAGE_ENC)
         buf[0] |= 0x04 | 0x08;
     build_sig_subpkt (sig, SIGSUBPKT_KEY_FLAGS, buf, 1);
@@ -152,14 +157,23 @@ keygen_add_key_expire( PKT_signature *sig, void *opaque )
     u32  u;
 
     if( pk->expiredate ) {
-	u = pk->expiredate > pk->timestamp? pk->expiredate - pk->timestamp
-					  : pk->timestamp;
+        if(pk->expiredate > pk->timestamp)
+	  u= pk->expiredate - pk->timestamp;
+	else
+	  u= 0;
+
 	buf[0] = (u >> 24) & 0xff;
 	buf[1] = (u >> 16) & 0xff;
 	buf[2] = (u >>	8) & 0xff;
 	buf[3] = u & 0xff;
 	build_sig_subpkt( sig, SIGSUBPKT_KEY_EXPIRE, buf, 4 );
     }
+    else
+      {
+	/* Make sure we don't leave a key expiration subpacket lying
+	   around */
+	delete_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_EXPIRE);
+      }
 
     return 0;
 }
@@ -219,9 +233,9 @@ keygen_set_std_prefs (const char *string,int personal)
       if (opt.def_preference_list)
 	string=opt.def_preference_list;
       else if ( !check_cipher_algo(CIPHER_ALGO_IDEA) )
-        string = "S7 S3 S2 S1 H2 H3 Z2 Z1";
+        string = "S9 S8 S7 S3 S2 S1 H2 H3 Z2 Z1";
       else
-        string = "S7 S3 S2 H2 H3 Z2 Z1";
+        string = "S9 S8 S7 S3 S2 H2 H3 Z2 Z1";
 
       /* If we have it, IDEA goes *after* 3DES so it won't be used
          unless we're encrypting along with a V3 key.  Ideally, we
@@ -235,17 +249,17 @@ keygen_set_std_prefs (const char *string,int personal)
         string = "";
 
     for (s=string; *s; s = s2) {
-        if ((*s=='s' || *s == 'S') && isdigit(s[1]) ) {
+        if ((*s=='s' || *s == 'S') && digitp (s+1) ) {
             val = strtoul (++s, (char**)&s2, 10);
             if (set_one_pref (val, 'S', check_cipher_algo, sym, &nsym))
                 rc = -1;
         }
-        else if ((*s=='h' || *s == 'H') && isdigit(s[1]) ) {
+        else if ((*s=='h' || *s == 'H') && digitp (s+1) ) {
             val = strtoul (++s, (char**)&s2, 10);
             if (set_one_pref (val, 'H', check_digest_algo, hash, &nhash))
                 rc = -1;
         }
-        else if ((*s=='z' || *s == 'Z') && isdigit(s[1]) ) {
+        else if ((*s=='z' || *s == 'Z') && digitp (s+1) ) {
             val = strtoul (++s, (char**)&s2, 10);
             if (set_one_pref (val, 'Z', check_compress_algo, zip, &nzip))
                 rc = -1;
@@ -258,7 +272,7 @@ keygen_set_std_prefs (const char *string,int personal)
 	  mdc=0;
 	  s2=s+6;
 	}
-        else if (isspace (*s))
+        else if (spacep (s))
             s2 = s+1;
         else {
             log_info (_("invalid character in preference string\n"));
@@ -906,10 +920,10 @@ gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 static int
 check_valid_days( const char *s )
 {
-    if( !isdigit(*s) )
+    if( !digitp(s) )
 	return 0;
     for( s++; *s; s++)
-	if( !isdigit(*s) )
+	if( !digitp(s) )
 	    break;
     if( !*s )
 	return 1;
@@ -944,13 +958,11 @@ ask_algo (int addmode, unsigned int *r_usage)
     tty_printf(    _("   (%d) DSA (sign only)\n"), 2 );
     if( addmode )
 	tty_printf(    _("   (%d) ElGamal (encrypt only)\n"), 3 );
-    if (opt.expert)
-        tty_printf(    _("   (%d) ElGamal (sign and encrypt)\n"), 4 );
-    tty_printf(    _("   (%d) RSA (sign only)\n"), 5 );
+    tty_printf(    _("   (%d) RSA (sign only)\n"), 4 );
     if (addmode)
-        tty_printf(    _("   (%d) RSA (encrypt only)\n"), 6 );
+        tty_printf(    _("   (%d) RSA (encrypt only)\n"), 5 );
     if (opt.expert)
-      tty_printf(    _("   (%d) RSA (sign and encrypt)\n"), 7 );
+      tty_printf(    _("   (%d) RSA (sign and encrypt)\n"), 6 );
 
     for(;;) {
 	answer = cpr_get("keygen.algo",_("Your selection? "));
@@ -961,37 +973,29 @@ ask_algo (int addmode, unsigned int *r_usage)
 	    algo = 0;	/* create both keys */
 	    break;
 	}
-	else if( algo == 7 && opt.expert ) {
-	    if (cpr_get_answer_is_yes ("keygen.algo.rsa_se",_(
-		"The use of this algorithm is deprecated - create anyway? "))){
-              algo = PUBKEY_ALGO_RSA;
-              *r_usage = PUBKEY_USAGE_ENC | PUBKEY_USAGE_SIG;
-              break;
-            }
+	else if( algo == 6 && opt.expert ) {
+	    algo = PUBKEY_ALGO_RSA;
+	    *r_usage = PUBKEY_USAGE_ENC | PUBKEY_USAGE_SIG;
+	    break;
 	}
-	else if( algo == 6 && addmode ) {
+	else if( algo == 5 && addmode ) {
 	    algo = PUBKEY_ALGO_RSA;
             *r_usage = PUBKEY_USAGE_ENC;
 	    break;
 	}
-	else if( algo == 5 ) {
+	else if( algo == 4 ) {
 	    algo = PUBKEY_ALGO_RSA;
             *r_usage = PUBKEY_USAGE_SIG;
 	    break;
 	}
-	else if( algo == 4 && opt.expert) {
-	    if( cpr_get_answer_is_yes("keygen.algo.elg_se",_(
-		"The use of this algorithm is deprecated - create anyway? "))){
-		algo = PUBKEY_ALGO_ELGAMAL;
-		break;
-	    }
-	}
 	else if( algo == 3 && addmode ) {
 	    algo = PUBKEY_ALGO_ELGAMAL_E;
+            *r_usage = PUBKEY_USAGE_ENC;
 	    break;
 	}
 	else if( algo == 2 ) {
 	    algo = PUBKEY_ALGO_DSA;
+            *r_usage = PUBKEY_USAGE_SIG;
 	    break;
 	}
 	else
@@ -1239,7 +1243,7 @@ ask_user_id( int mode )
 
 		if( strpbrk( aname, "<>" ) )
 		    tty_printf(_("Invalid character in name\n"));
-		else if( isdigit(*aname) )
+		else if( digitp(aname) )
 		    tty_printf(_("Name may not start with a digit\n"));
 		else if( strlen(aname) < 5 )
 		    tty_printf(_("Name must be at least 5 characters long\n"));
@@ -1253,7 +1257,7 @@ ask_user_id( int mode )
 		amail = cpr_get("keygen.email",_("Email address: "));
 		trim_spaces(amail);
 		cpr_kill_prompt();
-		if( !*amail )
+		if( !*amail || opt.allow_freeform_uid )
 		    break;   /* no email address is okay */
 		else if( has_invalid_email_chars(amail)
 			 || string_count_chr(amail,'@') != 1
@@ -1306,11 +1310,13 @@ ask_user_id( int mode )
 
 	tty_printf(_("You selected this USER-ID:\n    \"%s\"\n\n"), uid);
 	/* fixme: add a warning if this user-id already exists */
-	if( !*amail && (strchr( aname, '@' ) || strchr( acomment, '@'))) {
+	if( !*amail && !opt.allow_freeform_uid
+	    && (strchr( aname, '@' ) || strchr( acomment, '@')))
+	  {
 	    fail = 1;
 	    tty_printf(_("Please don't put the email address "
-			  "into the real name or the comment\n") );
-	}
+			 "into the real name or the comment\n") );
+	  }
 
 	for(;;) {
 	    const char *ansstr = _("NnCcEeOoQq");
@@ -1388,10 +1394,11 @@ ask_passphrase( STRING2KEY **ret_s2k )
     for(;;) {
 	s2k->mode = opt.s2k_mode;
 	s2k->hash_algo = opt.s2k_digest_algo;
-	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k,2,errtext);
+	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k,2,
+                                 errtext, NULL);
 	if( !dek ) {
-	    errtext = _("passphrase not correctly repeated; try again");
-	    tty_printf(_("%s.\n"), errtext);
+             errtext = N_("passphrase not correctly repeated; try again");
+             tty_printf(_("%s.\n"), _(errtext));
 	}
 	else if( !dek->keylen ) {
 	    m_free(dek); dek = NULL;
@@ -1432,7 +1439,7 @@ do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
     else
 	BUG();
 
-  #ifdef ENABLE_COMMENT_PACKETS
+#ifdef ENABLE_COMMENT_PACKETS
     if( !rc ) {
 	add_kbnode( pub_root,
 		make_comment_node("#created by GNUPG v" VERSION " ("
@@ -1441,7 +1448,7 @@ do_create( int algo, unsigned int nbits, KBNODE pub_root, KBNODE sec_root,
 		make_comment_node("#created by GNUPG v" VERSION " ("
 					    PRINTABLE_OS_NAME ")"));
     }
-  #endif
+#endif
     return rc;
 }
 
@@ -1508,7 +1515,7 @@ get_parameter_algo( struct para_data_s *para, enum para_name key )
     struct para_data_s *r = get_parameter( para, key );
     if( !r )
 	return -1;
-    if( isdigit( *r->u.value ) )
+    if( digitp( r->u.value ) )
 	i = atoi( r->u.value );
     else
         i = string_to_pubkey_algo( r->u.value );
@@ -1718,7 +1725,8 @@ proc_parameter_file( struct para_data_s *para, const char *fname,
 	s2k->mode = opt.s2k_mode;
 	s2k->hash_algo = opt.s2k_digest_algo;
 	set_next_passphrase( r->u.value );
-	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2, NULL );
+	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2,
+                                 NULL, NULL);
 	set_next_passphrase( NULL );
 	assert( dek );
 	memset( r->u.value, 0, strlen(r->u.value) );
@@ -1988,6 +1996,11 @@ generate_keypair( const char *fname )
 	strcpy( r->u.value, "1024" );
 	r->next = para;
 	para = r;
+	r = m_alloc_clear( sizeof *r + 20 );
+	r->key = pKEYUSAGE;
+	strcpy( r->u.value, "sign" );
+	r->next = para;
+	para = r;
 
 	algo = PUBKEY_ALGO_ELGAMAL_E;
 	r = m_alloc_clear( sizeof *r + 20 );
@@ -1995,6 +2008,11 @@ generate_keypair( const char *fname )
 	sprintf( r->u.value, "%d", algo );
 	r->next = para;
 	para = r;
+	r = m_alloc_clear( sizeof *r + 20 );
+	r->key = pSUBKEYUSAGE;
+	strcpy( r->u.value, "encrypt" );
+	r->next = para;
+	para = r;
     }
     else {
 	r = m_alloc_clear( sizeof *r + 20 );
@@ -2385,7 +2403,8 @@ generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock )
 	s2k->mode = opt.s2k_mode;
 	s2k->hash_algo = opt.s2k_digest_algo;
 	set_next_passphrase( passphrase );
-	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2, NULL );
+	dek = passphrase_to_dek( NULL, 0, opt.s2k_cipher_algo, s2k, 2,
+                                 NULL , NULL);
     }
 
     rc = do_create( algo, nbits, pub_keyblock, sec_keyblock,
